@@ -27,6 +27,8 @@
 #include <dbus/dbus-glib-lowlevel.h>
 #include <dbus/dbus-glib.h>
 
+#define TRACKER_TYPE_STR_STRV_MAP (dbus_g_type_get_map ("GHashTable", G_TYPE_STRING, G_TYPE_STRV))
+
 G_BEGIN_DECLS
 
 #if !defined (__LIBTRACKER_COMMON_INSIDE__) && !defined (TRACKER_COMPILATION)
@@ -129,6 +131,9 @@ void		 tracker_dbus_request_failed	     (gint			  request_id,
 void		 tracker_dbus_request_comment	     (gint			  request_id,
 						      const gchar		 *format,
 						      ...);
+void		 tracker_dbus_request_info	     (gint			  request_id,
+						      const gchar		 *format,
+						      ...);
 void		 tracker_dbus_request_debug	     (gint			  request_id,
 						      const gchar		 *format,
 						      ...);
