@@ -30,8 +30,10 @@
 #include <sys/types.h>
 #include <sys/stat.h>
 #include <sys/statvfs.h>
+#include <sys/file.h>
 #include <fcntl.h>
 #include <limits.h>
+#include <errno.h>
 
 #include <glib.h>
 #include <gio/gio.h>
@@ -43,6 +45,8 @@
 
 #define TEXT_SNIFF_SIZE 4096
 
+static GHashTable *file_locks = NULL;
+
 FILE *
 tracker_file_open (const gchar *uri,
 		   const gchar *how,
@@ -687,3 +691,140 @@ tracker_env_check_xdg_dirs (void)
 
 	return success;
 }
+
+gboolean
+tracker_file_lock (GFile *file)
+{
+	gint fd, retval;
+	gchar *path;
+
+	g_return_val_if_fail (G_IS_FILE (file), FALSE);
+
+	if (G_UNLIKELY (!file_locks)) {
+		file_locks = g_hash_table_new_full ((GHashFunc) g_file_hash,
+						    (GEqualFunc) g_file_equal,
+						    (GDestroyNotify) g_object_unref,
+						    NULL);
+	}
+
+	/* Don't try to lock twice */
+	if (g_hash_table_lookup (file_locks, file) != NULL) {
+		return TRUE;
+	}
+
+	if (!g_file_is_native (file)) {
+		return FALSE;
+	}
+
+	path = g_file_get_path (file);
+
+	if (!path) {
+		return FALSE;
+	}
+
+	fd = open (path, O_RDONLY);
+
+	if (fd < 0) {
+ 		g_warning ("Could not open '%s'", path);
+		g_free (path);
+
+		return FALSE;
+	}
+
+	retval = flock (fd, LOCK_EX);
+
+	if (retval == 0) {
+		g_hash_table_insert (file_locks,
+				     g_object_ref (file),
+				     GINT_TO_POINTER (fd));
+	} else {
+		g_warning ("Could not lock file '%s'", path);
+ 		close (fd);
+	}
+
+	g_free (path);
+
+	return (retval == 0);
+}
+
+gboolean
+tracker_file_unlock (GFile *file)
+{
+	gint retval, fd;
+
+	g_return_val_if_fail (G_IS_FILE (file), TRUE);
+
+	if (!file_locks) {
+		return TRUE;
+ 	}
+
+	fd = GPOINTER_TO_INT (g_hash_table_lookup (file_locks, file));
+
+	if (fd == 0) {
+		/* File wasn't actually locked */
+		return TRUE;
+	}
+
+	retval = flock (fd, LOCK_UN);
+
+	if (retval < 0) {
+		gchar *path;
+
+		path = g_file_get_path (file);
+		g_warning ("Could not unlock file '%s'", path);
+		g_free (path);
+
+		return FALSE;
+	}
+
+	g_hash_table_remove (file_locks, file);
+	close (fd);
+
+	return TRUE;
+}
+
+gboolean
+tracker_file_is_locked (GFile *file)
+{
+	gboolean retval = FALSE;
+	gchar *path;
+	gint fd;
+
+	g_return_val_if_fail (G_IS_FILE (file), FALSE);
+
+	if (!g_file_is_native (file)) {
+		return FALSE;
+	}
+
+	path = g_file_get_path (file);
+
+	if (!path) {
+		return FALSE;
+	}
+
+	fd = open (path, O_RDONLY);
+
+	if (fd < 0) {
+		g_warning ("Could not open '%s'", path);
+		g_free (path);
+
+		return FALSE;
+	}
+
+	/* Check for locks */
+	retval = flock (fd, LOCK_SH | LOCK_NB);
+
+	if (retval < 0) {
+		if (errno == EWOULDBLOCK) {
+			retval = TRUE;
+		}
+	} else {
+		/* Oops, call was successful, unlock again the file */
+		flock (fd, LOCK_UN);
+	}
+
+	close (fd);
+	g_free (path);
+
+	return retval;
+}
