@@ -46,7 +46,7 @@ typedef struct {
 	LibHalContext *context;
 	DBusConnection *connection;
 
-	GHashTable    *all_devices;
+	GHashTable *all_devices;
 
 	GNode *mounts;
 	GHashTable *mounts_by_udi;
@@ -66,8 +66,8 @@ typedef struct {
 
 typedef struct {
 	LibHalContext *context;
-	GList	      *roots;
-	gboolean       only_removable;
+	GSList *roots;
+	gboolean only_removable;
 } GetRoots;
 
 static void	tracker_storage_finalize	(GObject	 *object);
@@ -144,7 +144,7 @@ static void
 tracker_storage_init (TrackerStorage *storage)
 {
 	TrackerStoragePriv *priv;
-	DBusError	error;
+	DBusError error;
 
 	g_message ("Initializing HAL Storage...");
 
@@ -213,8 +213,8 @@ tracker_storage_init (TrackerStorage *storage)
 }
 
 static gboolean
-free_mount_info (GNode *node,
-		 gpointer user_data)
+free_mount_info (GNode    *node,
+		 gpointer  user_data)
 {
 	MountInfo *info;
 
@@ -296,10 +296,10 @@ hal_get_property (GObject    *object,
 static gboolean
 hal_setup_devices (TrackerStorage *storage)
 {
-	TrackerStoragePriv	*priv;
-	DBusError	 error;
-	gchar	       **devices, **p;
-	gint		 num;
+	TrackerStoragePriv *priv;
+	DBusError error;
+	gchar **devices, **p;
+	gint num;
 
 	priv = GET_PRIV (storage);
 
@@ -527,12 +527,13 @@ hal_drive_type_to_string (LibHalDriveType type)
 }
 
 static gboolean
-hal_device_is_removable (TrackerStorage *storage,
-			 const gchar    *device_file)
+hal_device_is_user_removable (TrackerStorage *storage,
+                              const gchar    *device_file,
+                              const gchar    *mount_point)
 {
-	TrackerStoragePriv	*priv;
-	LibHalDrive	*drive;
-	gboolean	 removable;
+	TrackerStoragePriv *priv;
+	LibHalDrive *drive;
+	gboolean removable;
 
 	if (!device_file) {
 		return FALSE;
@@ -546,6 +547,18 @@ hal_device_is_removable (TrackerStorage *storage,
 	}
 
 	removable = libhal_drive_uses_removable_media (drive);
+
+	if (libhal_drive_get_type (drive) == LIBHAL_DRIVE_TYPE_SD_MMC) {
+		/* mmc block devices are not considered removable according to
+		   linux kernel as they do not contain removable media, they
+		   are simply hotpluggable
+		   consider all SD/MMC volumes mounted in /media as user removable
+		 */
+		if (g_str_has_prefix (mount_point, "/media/")) {
+			removable = TRUE;
+		}
+	}
+
 	libhal_drive_free (drive);
 
 	return removable;
@@ -555,10 +568,10 @@ static gboolean
 hal_device_should_be_tracked (TrackerStorage *storage,
 			      const gchar    *device_file)
 {
-	TrackerStoragePriv	*priv;
-	LibHalDrive	*drive;
-	LibHalDriveType  drive_type;
-	gboolean	 eligible;
+	TrackerStoragePriv *priv;
+	LibHalDrive *drive;
+	LibHalDriveType drive_type;
+	gboolean eligible;
 
 	if (!device_file) {
 		return FALSE;
@@ -623,10 +636,10 @@ hal_device_add (TrackerStorage *storage,
 		LibHalVolume   *volume)
 {
 	TrackerStoragePriv *priv;
-	DBusError	error;
-	const gchar    *udi;
-	const gchar    *mount_point;
-	const gchar    *device_file;
+	DBusError error;
+	const gchar *udi;
+	const gchar *mount_point;
+	const gchar *device_file;
 
 	priv = GET_PRIV (storage);
 
@@ -665,7 +678,7 @@ hal_device_add (TrackerStorage *storage,
 		hal_mount_point_add (storage,
 				     udi,
 				     mount_point,
-				     hal_device_is_removable (storage, device_file));
+				     hal_device_is_user_removable (storage, device_file, mount_point));
 	}
 
 	return TRUE;
@@ -676,7 +689,7 @@ hal_device_added_cb (LibHalContext *context,
 		     const gchar   *udi)
 {
 	TrackerStorage *storage;
-	LibHalVolume   *volume;
+	LibHalVolume *volume;
 
 	storage = libhal_ctx_get_user_data (context);
 
@@ -712,11 +725,11 @@ static void
 hal_device_removed_cb (LibHalContext *context,
 		       const gchar   *udi)
 {
-	TrackerStorage     *storage;
+	TrackerStorage *storage;
 	TrackerStoragePriv *priv;
-	const gchar        *device_file;
+	const gchar *device_file;
 
-	storage = (TrackerStorage*) libhal_ctx_get_user_data (context);
+	storage = libhal_ctx_get_user_data (context);
 	priv = GET_PRIV (storage);
 
 	if (g_hash_table_lookup (priv->all_devices, udi)) {
@@ -745,11 +758,11 @@ hal_device_property_modified_cb (LibHalContext *context,
 				 dbus_bool_t	is_removed,
 				 dbus_bool_t	is_added)
 {
-	TrackerStorage     *storage;
+	TrackerStorage *storage;
 	TrackerStoragePriv *priv;
-	DBusError	    error;
+	DBusError error;
 
-	storage = (TrackerStorage*) libhal_ctx_get_user_data (context);
+	storage = libhal_ctx_get_user_data (context);
 	priv = GET_PRIV (storage);
 
 	dbus_error_init (&error);
@@ -800,7 +813,7 @@ hal_device_property_modified_cb (LibHalContext *context,
 			hal_mount_point_add (storage,
 					     udi,
 					     mount_point,
-					     hal_device_is_removable (storage, device_file));
+					     hal_device_is_user_removable (storage, device_file, mount_point));
 
 			libhal_volume_free (volume);
 		} else {
@@ -821,7 +834,7 @@ hal_device_property_modified_cb (LibHalContext *context,
  * Returns: The newly created #TrackerStorage.
  **/
 TrackerStorage *
-tracker_storage_new ()
+tracker_storage_new (void)
 {
 	return g_object_new (TRACKER_TYPE_STORAGE, NULL);
 }
@@ -831,18 +844,29 @@ hal_get_mount_point_by_udi_foreach (gpointer key,
 				    gpointer value,
 				    gpointer user_data)
 {
-	GetRoots      *gr;
-	const gchar   *udi;
+	GetRoots *gr;
+	const gchar *udi;
 	GNode *node;
 	MountInfo *info;
 
-	gr = (GetRoots*) user_data;
+	gr = user_data;
 	udi = key;
 	node = value;
 	info = node->data;
 
 	if (!gr->only_removable || info->removable) {
-		gr->roots = g_list_prepend (gr->roots, g_strdup (info->mount_point));
+		gchar *normalized_mount_point;
+		gint len;
+		
+		normalized_mount_point = g_strdup (info->mount_point);
+		len = strlen (normalized_mount_point);
+		
+		/* Don't include trailing slashes */
+		if (len > 2 && normalized_mount_point[len - 1] == G_DIR_SEPARATOR) {
+			normalized_mount_point[len - 1] = '\0';
+		}
+		
+		gr->roots = g_slist_prepend (gr->roots, normalized_mount_point);
 	}
 }
 
@@ -850,16 +874,19 @@ hal_get_mount_point_by_udi_foreach (gpointer key,
  * tracker_storage_get_mounted_directory_roots:
  * @storage: A #TrackerStorage
  *
- * Returns a #Glist of strings containing the root directories for mounted devices.
- * Each element must be freed using g_free() and the list itself using g_list_free().
+ * Returns a #GSList of strings containing the root directories for
+ * mounted devices. 
+ *
+ * Each element must be freed using g_free() and the list itself using
+ * g_slist_free().
  *
  * Returns: The list of root directories.
  **/
-GList *
+GSList *
 tracker_storage_get_mounted_directory_roots (TrackerStorage *storage)
 {
 	TrackerStoragePriv *priv;
-	GetRoots	gr;
+	GetRoots gr;
 
 	g_return_val_if_fail (TRACKER_IS_STORAGE (storage), NULL);
 
@@ -873,23 +900,26 @@ tracker_storage_get_mounted_directory_roots (TrackerStorage *storage)
 			      hal_get_mount_point_by_udi_foreach,
 			      &gr);
 
-	return gr.roots;
+	return g_slist_reverse (gr.roots);
 }
 
 /**
  * tracker_storage_get_removable_device_roots:
  * @storage: A #TrackerStorage
  *
- * Returns a #GList of strings containing the root directories for removable devices.
- * Each element must be freed using g_free() and the list itself through g_list_free().
+ * Returns a #GSList of strings containing the root directories for
+ * removable devices. 
+ *
+ * Each element must be freed using g_free() and the list itself
+ * through g_slist_free().
  *
  * Returns: The list of root directories.
  **/
-GList *
+GSList *
 tracker_storage_get_removable_device_roots (TrackerStorage *storage)
 {
 	TrackerStoragePriv *priv;
-	GetRoots	gr;
+	GetRoots gr;
 
 	g_return_val_if_fail (TRACKER_IS_STORAGE (storage), NULL);
 
@@ -903,7 +933,7 @@ tracker_storage_get_removable_device_roots (TrackerStorage *storage)
 			      hal_get_mount_point_by_udi_foreach,
 			      &gr);
 
-	return gr.roots;
+	return g_slist_reverse (gr.roots);
 }
 
 /**
@@ -926,9 +956,9 @@ tracker_storage_uri_is_on_removable_device (TrackerStorage *storage,
 					    gboolean       *available)
 {
 	TrackerStoragePriv *priv;
-	gchar              *path;
-	GFile              *file;
-	MountInfo          *info;
+	gchar *path;
+	GFile *file;
+	MountInfo *info;
 
 	g_return_val_if_fail (TRACKER_IS_STORAGE (storage), FALSE);
 
@@ -970,28 +1000,29 @@ tracker_storage_uri_is_on_removable_device (TrackerStorage *storage,
 	return TRUE;
 }
 
-
 /**
  * tracker_storage_get_removable_device_udis:
  * @storage: A #TrackerStorage
  *
- * Returns a #GList of strings containing the UDI for removable devices.
+ * Returns a #GSList of strings containing the UDI for removable devices.
  * Each element is owned by the #GHashTable internally, the list
- * itself through should be freed using g_list_free().
+ * itself through should be freed using g_slist_free().
  *
  * Returns: The list of UDIs.
  **/
-GList *
+GSList *
 tracker_storage_get_removable_device_udis (TrackerStorage *storage)
 {
 	TrackerStoragePriv *priv;
 	GHashTableIter iter;
 	gpointer key, value;
-	GList *udis = NULL;
+	GSList *udis;
 
 	g_return_val_if_fail (TRACKER_IS_STORAGE (storage), NULL);
 
 	priv = GET_PRIV (storage);
+
+	udis = NULL;
 	
 	g_hash_table_iter_init (&iter, priv->mounts_by_udi);
 
@@ -1005,11 +1036,11 @@ tracker_storage_get_removable_device_udis (TrackerStorage *storage)
 		info = node->data;
 
 		if (info->removable) {
-			udis = g_list_prepend (udis, (gpointer) udi);
+			udis = g_slist_prepend (udis, g_strdup (udi));
 		}
 	}
 
-	return g_list_reverse (udis);
+	return g_slist_reverse (udis);
 }
 
 /**
@@ -1054,9 +1085,9 @@ tracker_storage_udi_get_is_mounted (TrackerStorage *storage,
 				    const gchar    *udi)
 {
 	TrackerStoragePriv *priv;
-	LibHalVolume   *volume;
-	const gchar    *mount_point;
-	gboolean        is_mounted;
+	LibHalVolume *volume;
+	const gchar *mount_point;
+	gboolean is_mounted;
 
 	g_return_val_if_fail (TRACKER_IS_STORAGE (storage), FALSE);
 	g_return_val_if_fail (udi != NULL, FALSE);
@@ -1080,8 +1111,6 @@ tracker_storage_udi_get_is_mounted (TrackerStorage *storage,
 
 }
 
-
-
 /**
  * tracker_storage_get_volume_udi_for_file:
  * @storage: A #TrackerStorage
@@ -1096,8 +1125,8 @@ tracker_storage_get_volume_udi_for_file (TrackerStorage *storage,
 					 GFile          *file)
 {
 	TrackerStoragePriv *priv;
-	gchar              *path;
-	MountInfo          *info;
+	gchar *path;
+	MountInfo *info;
 
 	g_return_val_if_fail (TRACKER_IS_STORAGE (storage), FALSE);
 
@@ -1121,4 +1150,5 @@ tracker_storage_get_volume_udi_for_file (TrackerStorage *storage,
 
 	return info->udi;
 }
+
 #endif /* HAVE_HAL */
