@@ -50,6 +50,8 @@ typedef struct _TrackerDataUpdateBufferPredicate TrackerDataUpdateBufferPredicat
 typedef struct _TrackerDataUpdateBufferProperty TrackerDataUpdateBufferProperty;
 typedef struct _TrackerDataUpdateBufferTable TrackerDataUpdateBufferTable;
 typedef struct _TrackerDataBlankBuffer TrackerDataBlankBuffer;
+typedef struct _TrackerStatementDelegate TrackerStatementDelegate;
+typedef struct _TrackerCommitDelegate TrackerCommitDelegate;
 
 struct _TrackerDataUpdateBuffer {
 	GHashTable *resource_cache;
@@ -93,6 +95,17 @@ struct _TrackerDataBlankBuffer {
 	gchar *subject;
 	GArray *predicates;
 	GArray *objects;
+	GArray *graphs;
+};
+
+struct _TrackerStatementDelegate {
+	TrackerStatementCallback callback;
+	gpointer user_data;
+};
+
+struct _TrackerCommitDelegate {
+	TrackerCommitCallback callback;
+	gpointer user_data;
 };
 
 static gboolean in_transaction = FALSE;
@@ -101,45 +114,73 @@ static TrackerDataUpdateBuffer update_buffer;
 static TrackerDataUpdateBufferResource *resource_buffer;
 static TrackerDataBlankBuffer blank_buffer;
 
-static TrackerStatementCallback insert_callback = NULL;
-static gpointer insert_data;
-static TrackerStatementCallback delete_callback = NULL;
-static gpointer delete_data;
-static TrackerCommitCallback commit_callback = NULL;
-static gpointer commit_data;
-static TrackerCommitCallback rollback_callback = NULL;
-static gpointer rollback_data;
+static GPtrArray *insert_callbacks = NULL;
+static GPtrArray *delete_callbacks = NULL;
+static GPtrArray *commit_callbacks = NULL;
+static GPtrArray *rollback_callbacks = NULL;
 
 void 
-tracker_data_set_commit_statement_callback (TrackerCommitCallback    callback,
+tracker_data_add_commit_statement_callback (TrackerCommitCallback    callback,
 					    gpointer                 user_data)
 {
-	commit_callback = callback;
-	commit_data = user_data;
+	TrackerCommitDelegate *delegate = g_new0 (TrackerCommitDelegate, 1);
+
+	if (!commit_callbacks) {
+		commit_callbacks = g_ptr_array_new ();
+	}
+
+	delegate->callback = callback;
+	delegate->user_data = user_data;
+
+	g_ptr_array_add (commit_callbacks, delegate);
 }
 
 void
-tracker_data_set_rollback_statement_callback (TrackerCommitCallback    callback,
+tracker_data_add_rollback_statement_callback (TrackerCommitCallback    callback,
 					      gpointer                 user_data)
 {
-	rollback_callback = callback;
-	rollback_data = user_data;
+	TrackerCommitDelegate *delegate = g_new0 (TrackerCommitDelegate, 1);
+
+	if (!rollback_callbacks) {
+		rollback_callbacks = g_ptr_array_new ();
+	}
+
+	delegate->callback = callback;
+	delegate->user_data = user_data;
+
+	g_ptr_array_add (rollback_callbacks, delegate);
 }
 
 void
-tracker_data_set_insert_statement_callback (TrackerStatementCallback callback,
+tracker_data_add_insert_statement_callback (TrackerStatementCallback callback,
 					    gpointer                 user_data)
 {
-	insert_callback = callback;
-	insert_data = user_data;
+	TrackerStatementDelegate *delegate = g_new0 (TrackerStatementDelegate, 1);
+
+	if (!insert_callbacks) {
+		insert_callbacks = g_ptr_array_new ();
+	}
+
+	delegate->callback = callback;
+	delegate->user_data = user_data;
+
+	g_ptr_array_add (insert_callbacks, delegate);
 }
 
 void 
-tracker_data_set_delete_statement_callback (TrackerStatementCallback callback,
+tracker_data_add_delete_statement_callback (TrackerStatementCallback callback,
 					    gpointer                 user_data)
 {
-	delete_callback = callback;
-	delete_data = user_data;
+	TrackerStatementDelegate *delegate = g_new0 (TrackerStatementDelegate, 1);
+
+	if (!delete_callbacks) {
+		delete_callbacks = g_ptr_array_new ();
+	}
+
+	delegate->callback = callback;
+	delegate->user_data = user_data;
+
+	g_ptr_array_add (delete_callbacks, delegate);
 }
 
 GQuark tracker_data_error_quark (void) {
@@ -378,7 +419,7 @@ statement_bind_gvalue (TrackerDBStatement *stmt,
 }
 
 static void
-tracker_data_resource_buffer_flush (void)
+tracker_data_resource_buffer_flush (GError **error)
 {
 	TrackerDBInterface             *iface;
 	TrackerDBStatement             *stmt;
@@ -388,6 +429,7 @@ tracker_data_resource_buffer_flush (void)
 	const gchar                    *table_name;
 	GString                        *sql, *fts;
 	int                             i;
+	GError                         *actual_error = NULL;
 
 	iface = tracker_db_manager_get_db_interface ();
 
@@ -397,11 +439,16 @@ tracker_data_resource_buffer_flush (void)
 			"UPDATE \"rdfs:Resource\" SET Uri = ? WHERE ID = ?");
 		tracker_db_statement_bind_text (stmt, 0, resource_buffer->new_subject);
 		tracker_db_statement_bind_int (stmt, 1, resource_buffer->id);
-		tracker_db_statement_execute (stmt, NULL);
+		tracker_db_statement_execute (stmt, &actual_error);
 		g_object_unref (stmt);
 
 		g_free (resource_buffer->new_subject);
 		resource_buffer->new_subject = NULL;
+
+		if (actual_error) {
+			g_propagate_error (error, actual_error);
+			return;
+		}
 	}
 
 	g_hash_table_iter_init (&iter, resource_buffer->tables);
@@ -425,8 +472,13 @@ tracker_data_resource_buffer_flush (void)
 
 				tracker_db_statement_bind_int (stmt, 0, resource_buffer->id);
 				statement_bind_gvalue (stmt, 1, &property->value);
-				tracker_db_statement_execute (stmt, NULL);
+				tracker_db_statement_execute (stmt, &actual_error);
 				g_object_unref (stmt);
+
+				if (actual_error) {
+					g_propagate_error (error, actual_error);
+					return;
+				}
 			}
 		} else {
 			if (table->delete_row) {
@@ -434,15 +486,25 @@ tracker_data_resource_buffer_flush (void)
 				stmt = tracker_db_interface_create_statement (iface, "DELETE FROM \"rdfs:Resource_rdf:type\" WHERE ID = ? AND \"rdf:type\" = ?");
 				tracker_db_statement_bind_int (stmt, 0, resource_buffer->id);
 				tracker_db_statement_bind_int (stmt, 1, ensure_resource_id (tracker_class_get_uri (table->class)));
-				tracker_db_statement_execute (stmt, NULL);
+				tracker_db_statement_execute (stmt, &actual_error);
 				g_object_unref (stmt);
 
+				if (actual_error) {
+					g_propagate_error (error, actual_error);
+					return;
+				}
+
 				/* remove row from class table */
 				stmt = tracker_db_interface_create_statement (iface, "DELETE FROM \"%s\" WHERE ID = ?", table_name);
 				tracker_db_statement_bind_int (stmt, 0, resource_buffer->id);
-				tracker_db_statement_execute (stmt, NULL);
+				tracker_db_statement_execute (stmt, &actual_error);
 				g_object_unref (stmt);
 
+				if (actual_error) {
+					g_propagate_error (error, actual_error);
+					return;
+				}
+
 				continue;
 			}
 
@@ -452,8 +514,13 @@ tracker_data_resource_buffer_flush (void)
 					"INSERT OR IGNORE INTO \"%s\" (ID) VALUES (?)",
 					table_name);
 				tracker_db_statement_bind_int (stmt, 0, resource_buffer->id);
-				tracker_db_statement_execute (stmt, NULL);
+				tracker_db_statement_execute (stmt, &actual_error);
 				g_object_unref (stmt);
+
+				if (actual_error) {
+					g_propagate_error (error, actual_error);
+					return;
+				}
 			}
 
 			if (table->properties->len == 0) {
@@ -486,10 +553,15 @@ tracker_data_resource_buffer_flush (void)
 				}
 			}
 
-			tracker_db_statement_execute (stmt, NULL);
+			tracker_db_statement_execute (stmt, &actual_error);
 			g_object_unref (stmt);
 
 			g_string_free (sql, TRUE);
+
+			if (actual_error) {
+				g_propagate_error (error, actual_error);
+				return;
+			}
 		}
 	}
 
@@ -532,13 +604,18 @@ static void resource_buffer_free (TrackerDataUpdateBufferResource *resource)
 }
 
 void
-tracker_data_update_buffer_flush (void)
+tracker_data_update_buffer_flush (GError **error)
 {
 	GHashTableIter iter;
+	GError *actual_error = NULL;
 
 	g_hash_table_iter_init (&iter, update_buffer.resources);
 	while (g_hash_table_iter_next (&iter, NULL, (gpointer*) &resource_buffer)) {
-		tracker_data_resource_buffer_flush ();
+		tracker_data_resource_buffer_flush (&actual_error);
+		if (actual_error) {
+			g_propagate_error (error, actual_error);
+			break;
+		}
 	}
 
 	g_hash_table_remove_all (update_buffer.resources);
@@ -556,7 +633,7 @@ tracker_data_update_buffer_clear (void)
 }
 
 static void
-tracker_data_blank_buffer_flush (void)
+tracker_data_blank_buffer_flush (GError **error)
 {
 	/* end of blank node */
 	gint i, id;
@@ -564,6 +641,7 @@ tracker_data_blank_buffer_flush (void)
 	gchar *blank_uri;
 	const gchar *sha1;
 	GChecksum *checksum;
+	GError *actual_error = NULL;
 
 	subject = blank_buffer.subject;
 	blank_buffer.subject = NULL;
@@ -592,27 +670,36 @@ tracker_data_blank_buffer_flush (void)
 		/* uri not found
 		   replay piled up statements to create resource */
 		for (i = 0; i < blank_buffer.predicates->len; i++) {
-			tracker_data_insert_statement (blank_uri,
-				g_array_index (blank_buffer.predicates, gchar *, i),
-				g_array_index (blank_buffer.objects, gchar *, i),
-				NULL);
+			tracker_data_insert_statement (g_array_index (blank_buffer.graphs, gchar *, i),
+			                               blank_uri,
+			                               g_array_index (blank_buffer.predicates, gchar *, i),
+			                               g_array_index (blank_buffer.objects, gchar *, i),
+			                               &actual_error);
+			if (actual_error) {
+				break;
+			}
 		}
 	}
 
 	/* free piled up statements */
 	for (i = 0; i < blank_buffer.predicates->len; i++) {
+		g_free (g_array_index (blank_buffer.graphs, gchar *, i));
 		g_free (g_array_index (blank_buffer.predicates, gchar *, i));
 		g_free (g_array_index (blank_buffer.objects, gchar *, i));
 	}
+	g_array_free (blank_buffer.graphs, TRUE);
 	g_array_free (blank_buffer.predicates, TRUE);
 	g_array_free (blank_buffer.objects, TRUE);
 
 	g_hash_table_insert (blank_buffer.table, subject, blank_uri);
 	g_checksum_free (checksum);
+
+	g_propagate_error (error, actual_error);
 }
 
 static void
-cache_create_service_decomposed (TrackerClass           *cl)
+cache_create_service_decomposed (TrackerClass *cl,
+                                 const gchar  *graph)
 {
 	TrackerClass       **super_classes;
 	GValue              gvalue = { 0 };
@@ -622,7 +709,7 @@ cache_create_service_decomposed (TrackerClass           *cl)
 	/* also create instance of all super classes */
 	super_classes = tracker_class_get_super_classes (cl);
 	while (*super_classes) {
-		cache_create_service_decomposed (*super_classes);
+		cache_create_service_decomposed (*super_classes, graph);
 		super_classes++;
 	}
 
@@ -645,9 +732,18 @@ cache_create_service_decomposed (TrackerClass           *cl)
 	cache_insert_value ("rdfs:Resource_rdf:type", "rdf:type", &gvalue, TRUE, FALSE);
 
 	tracker_class_set_count (cl, tracker_class_get_count (cl) + 1);
-	if (insert_callback) {
-		insert_callback (resource_buffer->subject, RDF_PREFIX "type", class_uri, 
-		                 resource_buffer->types, insert_data);
+	if (insert_callbacks) {
+		guint n;
+
+		for (n = 0; n < insert_callbacks->len; n++) {
+			TrackerStatementDelegate *delegate;
+
+			delegate = g_ptr_array_index (insert_callbacks, n);
+			delegate->callback (graph, resource_buffer->subject,
+			                    RDF_PREFIX "type", class_uri, 
+			                    resource_buffer->types, 
+			                    delegate->user_data);
+		}
 	}
 }
 
@@ -965,6 +1061,7 @@ delete_metadata_decomposed (TrackerProperty  *property,
 	TrackerProperty   **super_properties;
 	GValue gvalue = { 0 };
 	GValueArray        *old_values;
+	GError             *new_error = NULL;
 
 	multiple_values = tracker_property_get_multiple_values (property);
 	if (multiple_values) {
@@ -979,9 +1076,10 @@ delete_metadata_decomposed (TrackerProperty  *property,
 	fts = tracker_property_get_fulltext_indexed (property);
 
 	/* read existing property values */
-	old_values = get_old_property_values (property, error);
-	if (error && *error) {
+	old_values = get_old_property_values (property, &new_error);
+	if (new_error) {
 		g_free (table_name);
+		g_propagate_error (error, new_error);
 		return;
 	}
 
@@ -1005,7 +1103,8 @@ delete_metadata_decomposed (TrackerProperty  *property,
 }
 
 static void
-cache_delete_resource_type (TrackerClass *class)
+cache_delete_resource_type (TrackerClass *class,
+                            const gchar  *graph)
 {
 	TrackerDBInterface *iface;
 	TrackerDBStatement *stmt;
@@ -1044,7 +1143,7 @@ cache_delete_resource_type (TrackerClass *class)
 			gchar *class_uri;
 
 			tracker_db_result_set_get (result_set, 0, &class_uri, -1);
-			cache_delete_resource_type (tracker_ontology_get_class_by_uri (class_uri));
+			cache_delete_resource_type (tracker_ontology_get_class_by_uri (class_uri), graph);
 			g_free (class_uri);
 		} while (tracker_db_result_set_iter_next (result_set));
 
@@ -1098,16 +1197,27 @@ cache_delete_resource_type (TrackerClass *class)
 	cache_delete_row (class);
 
 	tracker_class_set_count (class, tracker_class_get_count (class) - 1);
-	if (delete_callback) {
-		delete_callback (resource_buffer->subject, RDF_PREFIX "type", tracker_class_get_uri (class), resource_buffer->types, delete_data);
+
+	if (delete_callbacks) {
+		guint n;
+		for (n = 0; n < delete_callbacks->len; n++) {
+			TrackerStatementDelegate *delegate;
+
+			delegate = g_ptr_array_index (delete_callbacks, n);
+			delegate->callback (graph, resource_buffer->subject, 
+		                            RDF_PREFIX "type", tracker_class_get_uri (class),
+		                            resource_buffer->types, delegate->user_data);
+		}
 	}
+
 }
 
 void
-tracker_data_delete_statement (const gchar            *subject,
-			       const gchar            *predicate,
-			       const gchar            *object,
-			       GError                **error)
+tracker_data_delete_statement (const gchar  *graph,
+                               const gchar  *subject,
+                               const gchar  *predicate,
+                               const gchar  *object,
+                               GError      **error)
 {
 	TrackerClass       *class;
 	TrackerProperty    *field;
@@ -1146,7 +1256,7 @@ tracker_data_delete_statement (const gchar            *subject,
 	if (object && g_strcmp0 (predicate, RDF_PREFIX "type") == 0) {
 		class = tracker_ontology_get_class_by_uri (object);
 		if (class != NULL) {
-			cache_delete_resource_type (class);
+			cache_delete_resource_type (class, graph);
 		} else {
 			g_set_error (error, TRACKER_DATA_ERROR, TRACKER_DATA_ERROR_UNKNOWN_CLASS,
 				     "Class '%s' not found in the ontology", object);
@@ -1160,36 +1270,55 @@ tracker_data_delete_statement (const gchar            *subject,
 				     "Property '%s' not found in the ontology", predicate);
 		}
 
-		if (delete_callback) {
-			delete_callback (subject, predicate, object, resource_buffer->types, delete_data);
+		if (delete_callbacks) {
+			guint n;
+			for (n = 0; n < delete_callbacks->len; n++) {
+				TrackerStatementDelegate *delegate;
+
+				delegate = g_ptr_array_index (delete_callbacks, n);
+				delegate->callback (graph, subject, predicate, object, 
+				                    resource_buffer->types, 
+				                    delegate->user_data);
+			}
 		}
 	}
 }
 
 static gboolean
-tracker_data_insert_statement_common (const gchar            *subject,
-				      const gchar            *predicate,
-				      const gchar            *object)
+tracker_data_insert_statement_common (const gchar            *graph,
+                                      const gchar            *subject,
+                                      const gchar            *predicate,
+                                      const gchar            *object,
+                                      GError                **error)
 {
 	if (g_str_has_prefix (subject, ":")) {
 		/* blank node definition
 		   pile up statements until the end of the blank node */
 		gchar *value;
+		GError *actual_error = NULL;
 
 		if (blank_buffer.subject != NULL) {
 			/* active subject in buffer */
 			if (strcmp (blank_buffer.subject, subject) != 0) {
 				/* subject changed, need to flush buffer */
-				tracker_data_blank_buffer_flush ();
+				tracker_data_blank_buffer_flush (&actual_error);
+
+				if (actual_error) {
+					g_propagate_error (error, actual_error);
+					return FALSE;
+				}
 			}
 		}
 
 		if (blank_buffer.subject == NULL) {
 			blank_buffer.subject = g_strdup (subject);
+			blank_buffer.graphs = g_array_sized_new (FALSE, FALSE, sizeof (char*), 4);
 			blank_buffer.predicates = g_array_sized_new (FALSE, FALSE, sizeof (char*), 4);
 			blank_buffer.objects = g_array_sized_new (FALSE, FALSE, sizeof (char*), 4);
 		}
 
+		value = g_strdup (graph);
+		g_array_append_val (blank_buffer.graphs, value);
 		value = g_strdup (predicate);
 		g_array_append_val (blank_buffer.predicates, value);
 		value = g_strdup (object);
@@ -1234,10 +1363,11 @@ tracker_data_insert_statement_common (const gchar            *subject,
 }
 
 void
-tracker_data_insert_statement (const gchar            *subject,
-			       const gchar            *predicate,
-			       const gchar            *object,
-			       GError                **error)
+tracker_data_insert_statement (const gchar            *graph,
+                               const gchar            *subject,
+                               const gchar            *predicate,
+                               const gchar            *object,
+                               GError                **error)
 {
 	TrackerProperty *property;
 
@@ -1249,12 +1379,12 @@ tracker_data_insert_statement (const gchar            *subject,
 	property = tracker_ontology_get_property_by_uri (predicate);
 	if (property != NULL) {
 		if (tracker_property_get_data_type (property) == TRACKER_PROPERTY_TYPE_RESOURCE) {
-			tracker_data_insert_statement_with_uri (subject, predicate, object, error);
+			tracker_data_insert_statement_with_uri (graph, subject, predicate, object, error);
 		} else {
-			tracker_data_insert_statement_with_string (subject, predicate, object, error);
+			tracker_data_insert_statement_with_string (graph, subject, predicate, object, error);
 		}
 	} else if (strcmp (predicate, TRACKER_PREFIX "uri") == 0) {
-		tracker_data_insert_statement_with_uri (subject, predicate, object, error);
+		tracker_data_insert_statement_with_uri (graph, subject, predicate, object, error);
 	} else {
 		g_set_error (error, TRACKER_DATA_ERROR, TRACKER_DATA_ERROR_UNKNOWN_PROPERTY,
 		             "Property '%s' not found in the ontology", predicate);
@@ -1262,10 +1392,11 @@ tracker_data_insert_statement (const gchar            *subject,
 }
 
 void
-tracker_data_insert_statement_with_uri (const gchar            *subject,
-					const gchar            *predicate,
-					const gchar            *object,
-					GError                **error)
+tracker_data_insert_statement_with_uri (const gchar            *graph,
+                                        const gchar            *subject,
+                                        const gchar            *predicate,
+                                        const gchar            *object,
+                                        GError                **error)
 {
 	GError          *actual_error = NULL;
 	TrackerClass    *class;
@@ -1299,7 +1430,12 @@ tracker_data_insert_statement_with_uri (const gchar            *subject,
 		if (blank_buffer.subject != NULL) {
 			if (strcmp (blank_buffer.subject, object) == 0) {
 				/* object still in blank buffer, need to flush buffer */
-				tracker_data_blank_buffer_flush ();
+				tracker_data_blank_buffer_flush (&actual_error);
+
+				if (actual_error) {
+					g_propagate_error (error, actual_error);
+					return;
+				}
 			}
 		}
 
@@ -1307,17 +1443,27 @@ tracker_data_insert_statement_with_uri (const gchar            *subject,
 
 		if (blank_uri != NULL) {
 			/* now insert statement referring to blank node */
-			tracker_data_insert_statement (subject, predicate, blank_uri, error);
+			tracker_data_insert_statement (graph, subject, predicate, blank_uri, &actual_error);
 
 			g_hash_table_remove (blank_buffer.table, object);
 
+			if (actual_error) {
+				g_propagate_error (error, actual_error);
+				return;
+			}
+
 			return;
 		} else {
 			g_critical ("Blank node '%s' not found", object);
 		}
 	}
 
-	if (!tracker_data_insert_statement_common (subject, predicate, object)) {
+	if (!tracker_data_insert_statement_common (graph, subject, predicate, object, &actual_error)) {
+		if (actual_error) {
+			g_propagate_error (error, actual_error);
+			return;
+		}
+
 		return;
 	}
 
@@ -1326,7 +1472,7 @@ tracker_data_insert_statement_with_uri (const gchar            *subject,
 		   cope with inference and insert blank rows */
 		class = tracker_ontology_get_class_by_uri (object);
 		if (class != NULL) {
-			cache_create_service_decomposed (class);
+			cache_create_service_decomposed (class, graph);
 		} else {
 			g_set_error (error, TRACKER_DATA_ERROR, TRACKER_DATA_ERROR_UNKNOWN_CLASS,
 				     "Class '%s' not found in the ontology", object);
@@ -1343,17 +1489,26 @@ tracker_data_insert_statement_with_uri (const gchar            *subject,
 			return;
 		}
 
-		if (insert_callback) {
-			insert_callback (subject, predicate, object, resource_buffer->types, insert_data);
+		if (insert_callbacks) {
+			guint n;
+			for (n = 0; n < insert_callbacks->len; n++) {
+				TrackerStatementDelegate *delegate;
+
+				delegate = g_ptr_array_index (insert_callbacks, n);
+				delegate->callback (graph, subject, predicate, object, 
+				                    resource_buffer->types, 
+				                    delegate->user_data);
+			}
 		}
 	}
 }
 
 void
-tracker_data_insert_statement_with_string (const gchar            *subject,
-					   const gchar            *predicate,
-					   const gchar            *object,
-					   GError                **error)
+tracker_data_insert_statement_with_string (const gchar            *graph,
+                                           const gchar            *subject,
+                                           const gchar            *predicate,
+                                           const gchar            *object,
+                                           GError                **error)
 {
 	GError          *actual_error = NULL;
 	TrackerProperty *property;
@@ -1374,7 +1529,12 @@ tracker_data_insert_statement_with_string (const gchar            *subject,
 		return;
 	}
 
-	if (!tracker_data_insert_statement_common (subject, predicate, object)) {
+	if (!tracker_data_insert_statement_common (graph, subject, predicate, object, &actual_error)) {
+		if (actual_error) {
+			g_propagate_error (error, actual_error);
+			return;
+		}
+
 		return;
 	}
 
@@ -1386,8 +1546,16 @@ tracker_data_insert_statement_with_string (const gchar            *subject,
 		return;
 	}
 
-	if (insert_callback) {
-		insert_callback (subject, predicate, object, resource_buffer->types, insert_data);
+	if (insert_callbacks) {
+		guint n;
+		for (n = 0; n < insert_callbacks->len; n++) {
+			TrackerStatementDelegate *delegate;
+
+			delegate = g_ptr_array_index (insert_callbacks, n);
+			delegate->callback (graph, subject, predicate, object, 
+					    resource_buffer->types, 
+					    delegate->user_data);
+		}
 	}
 }
 
@@ -1428,10 +1596,10 @@ tracker_data_update_enable_volume (const gchar *udi,
 	mount_path_file = g_file_new_for_path (mount_path);
 	mount_path_uri = g_file_get_uri (mount_path_file);
 
-	delete_q = g_strdup_printf ("DELETE { <%s> tracker:mountPoint ?d } WHERE { <%s> tracker:mountPoint ?d }", 
-				    removable_device_urn, removable_device_urn);
-	set_q = g_strdup_printf ("INSERT { <%s> a tracker:Volume; tracker:mountPoint <%s> }", 
-				 removable_device_urn, mount_path_uri);
+	delete_q = g_strdup_printf ("DELETE FROM <%s> { <%s> tracker:mountPoint ?d } WHERE { <%s> tracker:mountPoint ?d }", 
+	                            removable_device_urn, removable_device_urn, removable_device_urn);
+	set_q = g_strdup_printf ("INSERT INTO <%s> { <%s> a tracker:Volume; tracker:mountPoint <%s> }", 
+	                         removable_device_urn, removable_device_urn, mount_path_uri);
 
 	tracker_data_update_sparql (delete_q, &error);
 
@@ -1451,10 +1619,10 @@ tracker_data_update_enable_volume (const gchar *udi,
 	g_free (set_q);
 	g_free (delete_q);
 
-	delete_q = g_strdup_printf ("DELETE { <%s> tracker:isMounted ?d } WHERE { <%s> tracker:isMounted ?d }", 
-				    removable_device_urn, removable_device_urn);
-	set_q = g_strdup_printf ("INSERT { <%s> a tracker:Volume; tracker:isMounted true }", 
-				 removable_device_urn);
+	delete_q = g_strdup_printf ("DELETE FROM <%s> { <%s> tracker:isMounted ?d } WHERE { <%s> tracker:isMounted ?d }", 
+	                            removable_device_urn, removable_device_urn, removable_device_urn);
+	set_q = g_strdup_printf ("INSERT INTO <%s> { <%s> a tracker:Volume; tracker:isMounted true }", 
+	                         removable_device_urn, removable_device_urn);
 
 	tracker_data_update_sparql (delete_q, &error);
 
@@ -1490,9 +1658,10 @@ tracker_data_update_reset_volume (const gchar *uri)
 
 	mnow = time (NULL);
 	now_as_string = tracker_date_to_string (mnow);
-	delete_q = g_strdup_printf ("DELETE { <%s> tracker:unmountDate ?d } WHERE { <%s> tracker:unmountDate ?d }", uri, uri);
-	set_q = g_strdup_printf ("INSERT { <%s> a tracker:Volume; tracker:unmountDate \"%s\" }", 
-				 uri, now_as_string);
+	delete_q = g_strdup_printf ("DELETE FROM <%s> { <%s> tracker:unmountDate ?d } WHERE { <%s> tracker:unmountDate ?d }", 
+	                            uri, uri, uri);
+	set_q = g_strdup_printf ("INSERT INTO <%s> { <%s> a tracker:Volume; tracker:unmountDate \"%s\" }", 
+	                         uri, uri, now_as_string);
 
 	tracker_data_update_sparql (delete_q, &error);
 
@@ -1530,10 +1699,10 @@ tracker_data_update_disable_volume (const gchar *udi)
 
 	tracker_data_update_reset_volume (removable_device_urn);
 
-	delete_q = g_strdup_printf ("DELETE { <%s> tracker:isMounted ?d } WHERE { <%s> tracker:isMounted ?d }", 
-				    removable_device_urn, removable_device_urn);
-	set_q = g_strdup_printf ("INSERT { <%s> a tracker:Volume; tracker:isMounted false }", 
-				 removable_device_urn);
+	delete_q = g_strdup_printf ("DELETE FROM <%s> { <%s> tracker:isMounted ?d } WHERE { <%s> tracker:isMounted ?d }", 
+	                            removable_device_urn, removable_device_urn, removable_device_urn);
+	set_q = g_strdup_printf ("INSERT INTO <%s> { <%s> a tracker:Volume; tracker:isMounted false }", 
+	                         removable_device_urn, removable_device_urn);
 
 	tracker_data_update_sparql (delete_q, &error);
 
@@ -1578,8 +1747,11 @@ tracker_data_update_disable_all_volumes (void)
 	tracker_db_statement_execute (stmt, NULL);
 	g_object_unref (stmt);
 
-	delete_q = g_strdup_printf ("DELETE { ?o tracker:isMounted ?d } WHERE { ?o tracker:isMounted ?d  FILTER (?o != <"TRACKER_NON_REMOVABLE_MEDIA_DATASOURCE_URN"> ) }");
-	set_q = g_strdup_printf ("INSERT { ?o a tracker:Volume; tracker:isMounted false } WHERE { ?o a tracker:Volume FILTER (?o != <"TRACKER_NON_REMOVABLE_MEDIA_DATASOURCE_URN"> ) }");
+	/* The INTO and FROM uris are not really right, but it should be harmless:
+	 * we just want graph to be != NULL in tracker-store/tracker-writeback.c */
+
+	delete_q = g_strdup_printf ("DELETE FROM <"TRACKER_NON_REMOVABLE_MEDIA_DATASOURCE_URN"> { ?o tracker:isMounted ?d } WHERE { ?o tracker:isMounted ?d  FILTER (?o != <"TRACKER_NON_REMOVABLE_MEDIA_DATASOURCE_URN"> ) }");
+	set_q = g_strdup_printf ("INSERT INTO <"TRACKER_NON_REMOVABLE_MEDIA_DATASOURCE_URN"> { ?o a tracker:Volume; tracker:isMounted false } WHERE { ?o a tracker:Volume FILTER (?o != <"TRACKER_NON_REMOVABLE_MEDIA_DATASOURCE_URN"> ) }");
 
 	tracker_data_update_sparql (delete_q, &error);
 
@@ -1630,7 +1802,7 @@ tracker_data_commit_transaction (void)
 
 	in_transaction = FALSE;
 
-	tracker_data_update_buffer_flush ();
+	tracker_data_update_buffer_flush (NULL);
 
 	if (update_buffer.fts_ever_updated) {
 		tracker_fts_update_commit ();
@@ -1644,8 +1816,13 @@ tracker_data_commit_transaction (void)
 	g_hash_table_unref (update_buffer.resources);
 	g_hash_table_unref (update_buffer.resource_cache);
 
-	if (commit_callback) {
-		commit_callback (commit_data);
+	if (commit_callbacks) {
+		guint n;
+		for (n = 0; n < commit_callbacks->len; n++) {
+			TrackerCommitDelegate *delegate;
+			delegate = g_ptr_array_index (commit_callbacks, n);
+			delegate->callback (delegate->user_data);
+		}
 	}
 }
 
@@ -1678,7 +1855,9 @@ format_sql_value_as_string (GString         *sql,
  * annotations (non-embedded/user metadata) stored about the resource.
  */
 void
-tracker_data_delete_resource_description (const gchar *uri, GError **error)
+tracker_data_delete_resource_description (const gchar *graph, 
+                                          const gchar *uri, 
+                                          GError **error)
 {
 	TrackerDBInterface *iface;
 	TrackerDBStatement *stmt;
@@ -1687,7 +1866,7 @@ tracker_data_delete_resource_description (const gchar *uri, GError **error)
 	GString		   *sql;
 	TrackerProperty	  **properties, **property;
 	int		    i;
-	gboolean            first;
+	gboolean            first, bail_out = FALSE;
 	gint                resource_id;
 
 	/* We use result_sets instead of cursors here because it's possible
@@ -1770,16 +1949,22 @@ tracker_data_delete_resource_description (const gchar *uri, GError **error)
 
 				if (!tracker_property_get_multiple_values (*property)) {
 					gchar *value;
+					GError *new_error = NULL;
 
 					/* single value property, value in single_result_set */
 
 					tracker_db_result_set_get (single_result, i++, &value, -1);
 
 					if (value) {
-						tracker_data_delete_statement (uri, 
+						
+						tracker_data_delete_statement (graph, uri, 
 						                               tracker_property_get_uri (*property), 
 						                               value, 
-						                               error);
+						                               &new_error);
+						if (new_error) {
+							g_propagate_error (error, new_error);
+							bail_out = TRUE;
+						}
 						g_free (value);
 					}
 
@@ -1803,16 +1988,23 @@ tracker_data_delete_resource_description (const gchar *uri, GError **error)
 					if (multi_result) {
 						do {
 							gchar *value;
+							GError *new_error = NULL;
 
 							tracker_db_result_set_get (multi_result, 0, &value, -1);
 
-							tracker_data_delete_statement (uri, 
+							tracker_data_delete_statement (graph, uri, 
 							                               tracker_property_get_uri (*property), 
 							                               value,
-							                               NULL);
+							                               &new_error);
 
 							g_free (value);
 
+							if (new_error) {
+								g_propagate_error (error, new_error);
+								bail_out = TRUE;
+								break;
+							}
+
 						} while (tracker_db_result_set_iter_next (multi_result));
 
 						g_object_unref (multi_result);
@@ -1826,7 +2018,7 @@ tracker_data_delete_resource_description (const gchar *uri, GError **error)
 				g_object_unref (single_result);
 			}
 
-		} while (tracker_db_result_set_iter_next (result_set));
+		} while (!bail_out && tracker_db_result_set_iter_next (result_set));
 
 		g_object_unref (result_set);
 	}
@@ -1837,6 +2029,7 @@ void
 tracker_data_update_sparql (const gchar  *update,
 			    GError      **error)
 {
+	GError *actual_error = NULL;
 	TrackerDBInterface *iface;
 	TrackerSparqlQuery *sparql_query;
 
@@ -1848,15 +2041,23 @@ tracker_data_update_sparql (const gchar  *update,
 
 	tracker_db_interface_execute_query (iface, NULL, "SAVEPOINT sparql");
 
-	tracker_sparql_query_execute (sparql_query, error);
+	tracker_sparql_query_execute_update (sparql_query, FALSE, &actual_error);
 
-	if (*error) {
+	if (actual_error) {
 		tracker_data_update_buffer_clear ();
 		tracker_db_interface_execute_query (iface, NULL, "ROLLBACK TO sparql");
 
-		if (rollback_callback) {
-			rollback_callback (commit_data);
+		if (rollback_callbacks) {
+			guint n;
+			for (n = 0; n < rollback_callbacks->len; n++) {
+				TrackerCommitDelegate *delegate;
+				delegate = g_ptr_array_index (rollback_callbacks, n);
+				delegate->callback (delegate->user_data);
+			}
 		}
+
+		g_propagate_error (error, actual_error);
+		return;
 	}
 
 	tracker_db_interface_execute_query (iface, NULL, "RELEASE sparql");
@@ -1864,3 +2065,47 @@ tracker_data_update_sparql (const gchar  *update,
 	g_object_unref (sparql_query);
 }
 
+
+GPtrArray *
+tracker_data_update_sparql_blank (const gchar  *update,
+			          GError      **error)
+{
+	GError *actual_error = NULL;
+	TrackerDBInterface *iface;
+	TrackerSparqlQuery *sparql_query;
+	GPtrArray *blank_nodes;
+
+	g_return_val_if_fail (update != NULL, NULL);
+
+	iface = tracker_db_manager_get_db_interface ();
+
+	sparql_query = tracker_sparql_query_new_update (update);
+
+	tracker_db_interface_execute_query (iface, NULL, "SAVEPOINT sparql");
+
+	blank_nodes = tracker_sparql_query_execute_update (sparql_query, TRUE, &actual_error);
+
+	if (actual_error) {
+		tracker_data_update_buffer_clear ();
+		tracker_db_interface_execute_query (iface, NULL, "ROLLBACK TO sparql");
+
+		if (rollback_callbacks) {
+			guint n;
+			for (n = 0; n < rollback_callbacks->len; n++) {
+				TrackerCommitDelegate *delegate;
+				delegate = g_ptr_array_index (rollback_callbacks, n);
+				delegate->callback (delegate->user_data);
+			}
+		}
+
+		g_propagate_error (error, actual_error);
+		return NULL;
+	}
+
+	tracker_db_interface_execute_query (iface, NULL, "RELEASE sparql");
+
+	g_object_unref (sparql_query);
+
+	return blank_nodes;
+}
+
