@@ -1092,10 +1092,15 @@ tracker_db_manager_init (TrackerDBManagerFlags	flags,
 	gboolean	    need_reindex, did_copy = FALSE;
 	guint		    i;
 
+	/* First set defaults for return values */
 	if (first_time) {
 		*first_time = FALSE;
 	}
 
+	if (need_journal) {
+		*need_journal = FALSE;
+	}
+
 	if (initialized) {
 		return TRUE;
 	}
