@@ -299,7 +299,7 @@ parser_next (TrackerParser *parser,
 	start = NULL;
 	end = NULL;
 
-	for (p = parser->cursor; *p; p = g_utf8_next_char (p)) {
+	for (p = parser->cursor; *p && *p != '\0'; p = g_utf8_next_char (p)) {
 		TrackerParserWordType type;
 		gunichar	      c;
 
@@ -442,7 +442,7 @@ parser_next (TrackerParser *parser,
 		*byte_offset_start = start-parser->txt;
 		*byte_offset_end = *byte_offset_start + bytes;
 
-		parser->cursor = g_utf8_next_char (parser->txt + *byte_offset_end);
+		parser->cursor = parser->txt + *byte_offset_end;
 
 		processed_word = tracker_parser_process_word (parser, utf8, bytes, do_strip);
 
