@@ -35,14 +35,76 @@
 #include "tracker-results-window.h"
 #include "tracker-aligned-window.h"
 
-#define MUSIC_SEARCH    "SELECT ?urn ?type ?title ?belongs WHERE { ?urn a nmm:MusicPiece ; rdf:type ?type ; nfo:fileName ?title ; nfo:belongsToContainer ?belongs . ?urn fts:match \"%s*\" } OFFSET 0 LIMIT 500"
-#define PHOTO_SEARCH    "SELECT ?urn ?type ?title ?belongs WHERE { ?urn a nmm:Photo ; rdf:type ?type ; nfo:fileName ?title ; nfo:belongsToContainer ?belongs . ?urn fts:match \"%s*\" } OFFSET 0 LIMIT 500"
-#define VIDEO_SEARCH    "SELECT ?urn ?type ?title ?belongs WHERE { ?urn a nmm:Video ; rdf:type ?type ; nfo:fileName ?title ; nfo:belongsToContainer ?belongs . ?urn fts:match \"%s*\" } OFFSET 0 LIMIT 500"
-#define DOCUMENT_SEARCH "SELECT ?urn ?type ?title ?belongs WHERE { ?urn a nfo:PaginatedTextDocument ; rdf:type ?type ; nfo:fileName ?title ; nfo:belongsToContainer ?belongs . ?urn fts:match \"%s*\" } OFFSET 0 LIMIT 500"
-#define FOLDER_SEARCH   "SELECT ?urn ?type ?title ?belongs WHERE { ?urn a nfo:Folder ; rdf:type ?type ; nfo:fileName ?title ; nfo:belongsToContainer ?belongs . ?urn fts:match \"%s*\" } OFFSET 0 LIMIT 500"
+#define MAX_ITEMS 5
+
+#define MUSIC_QUERY    "SELECT ?urn ?title ?belongs fts:rank(?urn) WHERE { ?urn a nfo:Audio ; nfo:fileName ?title ; nfo:belongsToContainer ?belongs . ?urn fts:match \"%s*\" } ORDER BY DESC(fts:rank(?urn)) OFFSET 0 LIMIT %d"
+#define IMAGE_QUERY    "SELECT ?urn ?title ?belongs fts:rank(?urn) WHERE { ?urn a nfo:Image ; nfo:fileName ?title ; nfo:belongsToContainer ?belongs . ?urn fts:match \"%s*\" } ORDER BY DESC(fts:rank(?urn)) OFFSET 0 LIMIT %d"
+#define VIDEO_QUERY    "SELECT ?urn ?title ?belongs fts:rank(?urn) WHERE { ?urn a nmm:Video ; nfo:fileName ?title ; nfo:belongsToContainer ?belongs . ?urn fts:match \"%s*\" } ORDER BY DESC(fts:rank(?urn)) OFFSET 0 LIMIT %d"
+#define DOCUMENT_QUERY "SELECT ?urn ?title ?belongs fts:rank(?urn) WHERE { ?urn a nfo:Document ; nfo:fileName ?title ; nfo:belongsToContainer ?belongs . ?urn fts:match \"%s*\" } ORDER BY DESC(fts:rank(?urn)) OFFSET 0 LIMIT %d"
+#define FOLDER_QUERY   "SELECT ?urn ?title ?belongs fts:rank(?urn) WHERE { ?urn a nfo:Folder ; nfo:fileName ?title ; nfo:belongsToContainer ?belongs . ?urn fts:match \"%s*\" } ORDER BY DESC(fts:rank(?urn)) OFFSET 0 LIMIT %d"
+#define APP_QUERY      "SELECT ?urn ?title WHERE { ?urn a nfo:Software ; nie:title ?title . FILTER regex (?title, \"%s\") } ORDER BY DESC(?title) OFFSET 0 LIMIT %d"
 
 #define GENERAL_SEARCH  "SELECT ?s ?type ?title WHERE { ?s fts:match \"%s*\" ; rdf:type ?type . OPTIONAL { ?s nie:title ?title } } OFFSET %d LIMIT %d"
 
+#undef USE_SEPARATOR_FOR_SPACING
+
+#define TRACKER_RESULTS_WINDOW_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TRACKER_TYPE_RESULTS_WINDOW, TrackerResultsWindowPrivate))
+
+typedef struct {
+	GtkWidget *frame;
+	GtkWidget *treeview;
+	GtkWidget *scrolled_window;
+	GObject *store;
+
+	GtkWidget *label;
+
+	GtkIconTheme *icon_theme;
+
+	TrackerClient *client;
+	gchar *query;
+
+	gboolean first_category_populated;
+
+	gint queries_pending;
+	gint request_id;
+} TrackerResultsWindowPrivate;
+
+typedef enum {
+	CATEGORY_NONE                  = 1 << 0,
+	CATEGORY_CONTACT               = 1 << 1,
+	CATEGORY_TAG                   = 1 << 2,
+	CATEGORY_EMAIL_ADDRESS         = 1 << 3,
+	CATEGORY_DOCUMENT              = 1 << 4,
+	CATEGORY_APPLICATION           = 1 << 5,
+	CATEGORY_IMAGE                 = 1 << 6,
+	CATEGORY_AUDIO                 = 1 << 7,
+	CATEGORY_FOLDER                = 1 << 8,
+	CATEGORY_FONT                  = 1 << 9,
+	CATEGORY_VIDEO                 = 1 << 10,
+	CATEGORY_ARCHIVE               = 1 << 11,
+	CATEGORY_WEBSITE               = 1 << 12
+} TrackerCategory;
+
+typedef struct {
+	gchar *urn;
+	gchar *type;
+	gchar *title;
+	gchar *belongs;
+	TrackerCategory category;
+} ItemData;
+
+typedef struct {
+	gint request_id;
+	TrackerCategory category;
+	TrackerResultsWindow *window;
+	GSList *results;
+} SearchQuery;
+
+struct FindCategory {
+	const gchar *category_str;
+	gboolean found;
+};
+
 static void     results_window_constructed        (GObject              *object);
 static void     results_window_finalize           (GObject              *object);
 static void     results_window_set_property       (GObject              *object,
@@ -63,41 +125,8 @@ static void     results_window_screen_changed     (GtkWidget            *widget,
 						   GdkScreen            *prev_screen);
 static void     model_set_up                      (TrackerResultsWindow *window);
 static void     search_get                        (TrackerResultsWindow *window,
-						   const gchar          *query);
-
-#define TRACKER_RESULTS_WINDOW_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), TRACKER_TYPE_RESULTS_WINDOW, TrackerResultsWindowPrivate))
-
-typedef struct {
-	GtkWidget *frame;
-	GtkWidget *treeview;
-	GtkWidget *scrolled_window;
-	GObject *store;
-
-	GtkWidget *label;
-
-	GtkIconTheme *icon_theme;
-
-	TrackerClient *client;
-	gchar *query;
-
-	gint queries_pending;
-} TrackerResultsWindowPrivate;
-
-typedef enum {
-	CATEGORY_NONE                  = 1 << 0,
-	CATEGORY_CONTACT               = 1 << 1,
-	CATEGORY_TAG                   = 1 << 2,
-	CATEGORY_EMAIL_ADDRESS         = 1 << 3,
-	CATEGORY_DOCUMENT              = 1 << 4,
-	CATEGORY_APPLICATION           = 1 << 5,
-	CATEGORY_IMAGE                 = 1 << 6,
-	CATEGORY_AUDIO                 = 1 << 7,
-	CATEGORY_FOLDER                = 1 << 8,
-	CATEGORY_FONT                  = 1 << 9,
-	CATEGORY_VIDEO                 = 1 << 10,
-	CATEGORY_ARCHIVE               = 1 << 11,
-	CATEGORY_WEBSITE               = 1 << 12
-} TrackerCategory;
+						   TrackerCategory       category);
+static void     search_start                      (TrackerResultsWindow *window);
 
 enum {
 	COL_CATEGORY_ID,
@@ -109,19 +138,6 @@ enum {
 	COL_COUNT
 };
 
-typedef struct {
-	gchar *urn;
-	gchar *type;
-	gchar *title;
-	gchar *belongs;
-	guint categories;
-} ItemData;
-
-struct FindCategory {
-	const gchar *category_str;
-	gboolean found;
-};
-
 enum {
 	PROP_0,
 	PROP_QUERY
@@ -151,7 +167,7 @@ tracker_results_window_class_init (TrackerResultsWindowClass *klass)
 							      "Query",
 							      "Query",
 							      NULL,
-							      G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+							      G_PARAM_READWRITE));
 
 	g_type_class_add_private (object_class, sizeof (TrackerResultsWindowPrivate));
 }
@@ -229,7 +245,7 @@ tracker_results_window_init (TrackerResultsWindow *window)
 	priv->frame = gtk_frame_new (NULL);
 	gtk_container_add (GTK_CONTAINER (window), priv->frame);
 	gtk_frame_set_shadow_type (GTK_FRAME (priv->frame), GTK_SHADOW_IN);
-	gtk_widget_set_size_request (priv->frame, 500, 300);
+	gtk_widget_set_size_request (priv->frame, 500, 600);
 	gtk_widget_show (priv->frame);
 
 	vbox = gtk_vbox_new (FALSE, 12);
@@ -263,32 +279,11 @@ tracker_results_window_init (TrackerResultsWindow *window)
 static void
 results_window_constructed (GObject *object)
 {
-	TrackerResultsWindowPrivate *priv;
 	TrackerResultsWindow *window;
-	gchar *sparql;
 
 	window = TRACKER_RESULTS_WINDOW (object);
-	priv = TRACKER_RESULTS_WINDOW_GET_PRIVATE (window);
 
-	sparql = g_strdup_printf (MUSIC_SEARCH, priv->query);
-	search_get (window, sparql);
-	g_free (sparql);
-
-	sparql = g_strdup_printf (PHOTO_SEARCH, priv->query);
-	search_get (window, sparql);
-	g_free (sparql);
-
-	sparql = g_strdup_printf (VIDEO_SEARCH, priv->query);
-	search_get (window, sparql);
-	g_free (sparql);
-
-	sparql = g_strdup_printf (DOCUMENT_SEARCH, priv->query);
-	search_get (window, sparql);
-	g_free (sparql);
-
-	sparql = g_strdup_printf (FOLDER_SEARCH, priv->query);
-	search_get (window, sparql);
-	g_free (sparql);
+	search_start (window);
 }
 
 static void
@@ -319,8 +314,12 @@ results_window_set_property (GObject      *object,
 
 	switch (prop_id) {
 	case PROP_QUERY:
+		/* Don't do the search_start() call if the window was
+		 * just set up. 
+		 */
 		g_free (priv->query);
 		priv->query = g_value_dup_string (value);
+		search_start (TRACKER_RESULTS_WINDOW (object));
 		break;
 	default:
                 G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -441,21 +440,19 @@ results_window_screen_changed (GtkWidget *widget,
 }
 
 static ItemData *
-item_data_new (const gchar *urn,
-	       const gchar *type,
-	       const gchar *title,
-	       const gchar *belongs,
-	       guint        categories)
+item_data_new (const gchar     *urn,
+	       const gchar     *title,
+	       const gchar     *belongs,
+	       TrackerCategory  category)
 {
 	ItemData *id;
 
 	id = g_slice_new0 (ItemData);
 
 	id->urn = g_strdup (urn);
-	id->type = g_strdup (type);
 	id->title = g_strdup (title);
 	id->belongs = g_strdup (belongs);
-	id->categories = categories;
+	id->category = category;
 
 	return id;
 }
@@ -464,21 +461,48 @@ static void
 item_data_free (ItemData *id)
 {
 	g_free (id->urn);
-	g_free (id->type);
 	g_free (id->title);
 	g_free (id->belongs);
 	
 	g_slice_free (ItemData, id);
 }
 
+static SearchQuery *
+search_query_new (gint                  request_id,
+		  TrackerCategory       category,
+		  TrackerResultsWindow *window)
+{
+	SearchQuery *sq;
+
+	sq = g_slice_new0 (SearchQuery);
+
+	sq->request_id = request_id;
+	sq->category = category;
+	sq->window = window;
+	sq->results = NULL;
+
+	return sq;
+}
+
+static void
+search_query_free (SearchQuery *sq)
+{
+	g_slist_foreach (sq->results, (GFunc) item_data_free, NULL);
+	g_slist_free (sq->results);
+
+	g_slice_free (SearchQuery, sq);
+}
+
 static gchar *
 category_to_string (TrackerCategory category)
 {
 	switch (category) {
+	case CATEGORY_NONE: return _("Other");
 	case CATEGORY_CONTACT: return _("Contacts");
 	case CATEGORY_TAG: return _("Tags");
 	case CATEGORY_EMAIL_ADDRESS: return _("Email Addresses");
 	case CATEGORY_DOCUMENT: return _("Documents");
+	case CATEGORY_APPLICATION: return _("Applications");
 	case CATEGORY_IMAGE: return _("Images");
 	case CATEGORY_AUDIO: return _("Audio");
 	case CATEGORY_FOLDER: return _("Folders");
@@ -486,9 +510,6 @@ category_to_string (TrackerCategory category)
 	case CATEGORY_VIDEO: return _("Videos");
 	case CATEGORY_ARCHIVE: return _("Archives");
 	case CATEGORY_WEBSITE: return _("Websites");
-
-	default:
-		break;
 	}
 
 	return _("Other");
@@ -640,23 +661,25 @@ model_category_cell_data_func (GtkTreeViewColumn    *tree_column,
 	GtkTreePath *path;
 	GtkTreeIter prev_iter;
 	TrackerCategory category, prev_category;
-	gboolean print = TRUE;
+	gboolean previous_path;
+	gboolean print = FALSE;
 
-	gtk_tree_model_get (model, iter,
-			    COL_CATEGORY_ID, &category,
-			    -1);
+	gtk_tree_model_get (model, iter, COL_CATEGORY_ID, &category, -1);
 
 	/* Get the previous iter */
 	path = gtk_tree_model_get_path (model, iter);
 
-	if (gtk_tree_path_prev (path) &&
-	    gtk_tree_model_get_iter (model, &prev_iter, path)) {
+	previous_path = gtk_tree_path_prev (path);
+
+	if (!previous_path) {
+		print = TRUE;
+	} else if (previous_path && gtk_tree_model_get_iter (model, &prev_iter, path)) {
 		gtk_tree_model_get (model, &prev_iter,
 				    COL_CATEGORY_ID, &prev_category,
 				    -1);
 
-		if (category == prev_category) {
-			print = FALSE;
+		if (prev_category == CATEGORY_NONE) {
+			print = TRUE;
 		}
 	}
 
@@ -689,9 +712,11 @@ model_pixbuf_cell_data_func (GtkTreeViewColumn    *tree_column,
 				    COL_URN, &urn,
 				    -1);
 
-		/* FIXME: Should use category */
-		pixbuf = pixbuf_get (window, urn, (category & CATEGORY_IMAGE));
-		g_free (urn);
+		if (urn) {
+			/* FIXME: Should use category */
+			pixbuf = pixbuf_get (window, urn, category);
+			g_free (urn);
+		}
 
 		/* Cache it in the store */
 		gtk_list_store_set (GTK_LIST_STORE (model), iter,
@@ -716,6 +741,50 @@ model_pixbuf_cell_data_func (GtkTreeViewColumn    *tree_column,
 	}
 }
 
+static gboolean
+model_separator_func (GtkTreeModel *model,
+		      GtkTreeIter  *iter,
+		      gpointer      user_data)
+{
+#ifdef USE_SEPARATOR_FOR_SPACING
+	gchar *urn;
+
+	gtk_tree_model_get (model, iter, COL_URN, &urn, -1);
+
+	if (!urn) {
+		return TRUE;
+	}
+
+	g_free (urn);
+
+	return FALSE;
+#else  /* USE_SEPARATOR_FOR_SPACING */
+	return FALSE;
+#endif /* USE_SEPARATOR_FOR_SPACING */
+}
+
+static gboolean
+model_selection_func (GtkTreeSelection *selection,
+		      GtkTreeModel     *model,
+		      GtkTreePath      *path,
+		      gboolean          path_currently_selected,
+		      gpointer          data)
+{
+	GtkTreeIter iter;
+	gchar *urn;
+
+	gtk_tree_model_get_iter (model, &iter, path);
+	gtk_tree_model_get (model, &iter, COL_URN, &urn, -1);
+
+	if (!urn) {
+		return FALSE;
+	}
+
+	g_free (urn);
+
+	return TRUE;
+}
+
 static void
 model_set_up (TrackerResultsWindow *window)
 {
@@ -729,6 +798,9 @@ model_set_up (TrackerResultsWindow *window)
 	priv = TRACKER_RESULTS_WINDOW_GET_PRIVATE (window);
 	view = GTK_TREE_VIEW (priv->treeview);
 
+	/* View */
+	gtk_tree_view_set_enable_search (view, FALSE);
+
 	/* Store */
 	store = gtk_list_store_new (COL_COUNT,
 				    G_TYPE_INT,            /* Category ID */
@@ -740,9 +812,18 @@ model_set_up (TrackerResultsWindow *window)
 
 	gtk_tree_view_set_model (view, GTK_TREE_MODEL (store));
 
+	gtk_tree_view_set_row_separator_func (view, 
+					      model_separator_func,
+					      window,
+					      NULL);
+
 	/* Selection */ 
 	selection = gtk_tree_view_get_selection (view);
 	gtk_tree_selection_set_mode (selection, GTK_SELECTION_SINGLE);
+        gtk_tree_selection_set_select_function (selection, 
+						model_selection_func,
+						window,
+						NULL);
 
 	/* Column: Category */
 	column = gtk_tree_view_column_new ();
@@ -786,9 +867,9 @@ model_set_up (TrackerResultsWindow *window)
 	gtk_tree_view_append_column (view, column);
 
 	/* Sorting */
-	gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (store),
-					      COL_CATEGORY_ID,
-					      GTK_SORT_ASCENDING);
+	/* gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (store), */
+	/* 				      COL_CATEGORY_ID, */
+	/* 				      GTK_SORT_ASCENDING); */
 
 	/* Tooltips */
 	gtk_tree_view_set_tooltip_column (view, COL_BELONGS);
@@ -849,6 +930,7 @@ search_window_ensure_not_blank (TrackerResultsWindow *window)
 			gtk_widget_show (priv->label);
 		} else {
 			gtk_widget_show_all (priv->scrolled_window);
+			gtk_widget_hide (priv->label);
 		}
 	}
 }
@@ -857,36 +939,39 @@ inline static void
 search_get_foreach (gpointer value, 
 		    gpointer user_data)
 {
-	GHashTable *resources;
+	SearchQuery *sq;
 	ItemData *id;
 	gchar **metadata;
-	const gchar *urn, *type, *title, *belongs;
+	const gchar *urn, *title, *belongs, *rank;
 
-	resources = user_data;
+	sq = user_data;
 	metadata = value;
 
 	urn = metadata[0];
-	type = metadata[1];
-	title = metadata[2];
-	belongs = metadata[3];
+	title = metadata[1];
+	belongs = metadata[2];
+
+	/* App queries don't return rank or belongs */
+	if (belongs) {
+		rank = metadata[3];
+	} else {
+		rank = "0.0";
+	}
 
 	if (!title) {
 		title = urn;
 	}
-	
-	id = g_hash_table_lookup (resources, urn);
-	if (!id) {
-		g_print ("urn:'%s' found\n", urn);
-		g_print ("  title:'%s'\n", title);
-		g_print ("  belongs to:'%s'\n", belongs);
 
-		id = item_data_new (urn, type, title, belongs, 0);
-		g_hash_table_insert (resources, g_strdup (urn), id);
-	}
+	g_print ("urn:'%s' found\n", urn);
+	g_print ("  title:'%s'\n", title);
+	g_print ("  belongs to:'%s'\n", belongs);
+	g_print ("  rank:'%s'\n", rank);
 
-	category_from_string (type, &id->categories);
+	id = item_data_new (urn, title, belongs, sq->category);
+	sq->results = g_slist_append (sq->results, id);
 
-	g_print ("  type:'%s', new categories:%d\n", type, id->categories);
+	/* category_from_string (type, &id->categories); */
+	/* g_print ("  type:'%s', new categories:%d\n", type, id->categories); */
 }
 
 static void
@@ -896,71 +981,155 @@ search_get_cb (GPtrArray *results,
 {
 	TrackerResultsWindow *window;
 	TrackerResultsWindowPrivate *priv;
+	SearchQuery *sq;
+
+	sq = user_data;
+	window = sq->window;
 
-	window = user_data;
 	priv = TRACKER_RESULTS_WINDOW_GET_PRIVATE (window);
 	priv->queries_pending--;
 
+	/* If request IDs don't match, data is no longer needed */
+	if (priv->request_id != sq->request_id) {
+		g_message ("Received data from request id:%d, now on request id:%d", 
+			   sq->request_id,
+			   priv->request_id);
+		search_query_free (sq);
+		return;
+	}
+
 	if (error) {
 		g_printerr ("Could not get search results, %s\n", error->message);
 		g_error_free (error);
-		
+
+		search_query_free (sq);
 		search_window_ensure_not_blank (window);
+
 		return;
 	}
 
 	if (!results) {
 		g_print ("No results were found matching the query\n");
 	} else {
-		GHashTable *resources;
-		GHashTableIter iter;
-		gpointer key, value;
+		GSList *l;
 
 		g_print ("Results: %d\n", results->len);
 
-		resources = g_hash_table_new_full (g_str_hash,
-						   g_str_equal,
-						   (GDestroyNotify) g_free,
-						   (GDestroyNotify) item_data_free);
+		if (results->len > 0) {
+			g_ptr_array_foreach (results,
+					     search_get_foreach,
+					     sq);
 
-		g_ptr_array_foreach (results,
-				     search_get_foreach,
-				     resources);
+			g_ptr_array_foreach (results, (GFunc) g_strfreev, NULL);
+			g_ptr_array_free (results, TRUE);
 
-		g_ptr_array_foreach (results, (GFunc) g_strfreev, NULL);
-		g_ptr_array_free (results, TRUE);
+			/* Add separator */
+			if (priv->first_category_populated) {
+				model_add (window, CATEGORY_NONE, NULL, NULL, NULL);
+			}
 
-		g_hash_table_iter_init (&iter, resources);
-		while (g_hash_table_iter_next (&iter, &key, &value)) {
-			ItemData *id;
+			for (l = sq->results; l; l = l->next) {
+				ItemData *id = l->data;
 
-			id = value;
+				model_add (window,
+					   sq->category,
+					   id->urn,
+					   id->title,
+					   id->belongs);
+			}
 
-			model_add (window,
-				   id->categories,
-				   id->urn,
-				   id->title,
-				   id->belongs);
+			priv->first_category_populated = TRUE;
 		}
-
-		g_hash_table_unref (resources);
 	}
 
+	search_query_free (sq);
 	search_window_ensure_not_blank (window);
 }
 
 static void
 search_get (TrackerResultsWindow *window,
-	    const gchar          *query)
+	    TrackerCategory       category)
 {
 	TrackerResultsWindowPrivate *priv;
+	SearchQuery *sq;
+	gchar *sparql;
+	const gchar *format;
 
 	priv = TRACKER_RESULTS_WINDOW_GET_PRIVATE (window);
 
-	tracker_resources_sparql_query_async (priv->client, query, search_get_cb, window);
+	switch (category) {
+	case CATEGORY_IMAGE:
+		format = IMAGE_QUERY;
+		break;
+	case CATEGORY_AUDIO:
+		format = MUSIC_QUERY;
+		break;
+	case CATEGORY_VIDEO:
+		format = VIDEO_QUERY;
+		break;
+	case CATEGORY_DOCUMENT:
+		format = DOCUMENT_QUERY;
+		break;
+	case CATEGORY_FOLDER:
+		format = FOLDER_QUERY;
+		break;
+	case CATEGORY_APPLICATION:
+		format = APP_QUERY;
+		break;
+	default:
+		format = NULL;
+		break;
+	}
+
+	if (!format) {
+		return;
+	}
+
+	sq = search_query_new (priv->request_id, category, window);
+
+	sparql = g_strdup_printf (format, priv->query, MAX_ITEMS);
+	tracker_resources_sparql_query_async (priv->client, sparql, search_get_cb, sq);
+	g_free (sparql);
+
 	priv->queries_pending++;
 }
 
+static void
+search_start (TrackerResultsWindow *window)
+{
+	TrackerResultsWindowPrivate *priv;
+	GtkTreeModel *model;
+	GtkListStore *store;
+
+	priv = TRACKER_RESULTS_WINDOW_GET_PRIVATE (window);
+	
+	/* Cancel current requests */
+	priv->request_id++;
+	g_message ("Incrementing request ID to %d", priv->request_id);
+
+	/* Clear current data */
+	g_message ("Clearing previous results");
+	model = gtk_tree_view_get_model (GTK_TREE_VIEW (priv->treeview));
+	store = GTK_LIST_STORE (model);
+	gtk_list_store_clear (store);
+
+	if (!priv->query || strlen (priv->query) < 1) {
+		gtk_widget_show (priv->scrolled_window);
+		gtk_widget_hide (priv->label);
+		return;
+	}
+
+	priv->first_category_populated = FALSE;
+
+	/* SPARQL requests */
+	search_get (window, CATEGORY_IMAGE);
+	search_get (window, CATEGORY_AUDIO);
+	search_get (window, CATEGORY_VIDEO);
+	search_get (window, CATEGORY_DOCUMENT);
+	search_get (window, CATEGORY_FOLDER);
+	search_get (window, CATEGORY_APPLICATION);
+}
+
 static gboolean
 grab_popup_window (TrackerResultsWindow *window)
 {
@@ -1014,5 +1183,8 @@ tracker_results_window_popup (TrackerResultsWindow *window)
 	gtk_widget_realize (GTK_WIDGET (window));
 	gtk_widget_show (GTK_WIDGET (window));
 
-	g_idle_add ((GSourceFunc) grab_popup_window, window);
+	if (0) {
+		g_idle_add ((GSourceFunc) grab_popup_window, window);
+	}
+
 }
