@@ -47,6 +47,21 @@ print_references_list (FILE *f, GList *list)
         g_fprintf (f,"</td>");
 }
 
+static void
+print_list (FILE *f, GList *list)
+{
+        GList *it;
+        gchar *shortname;
+
+
+        g_fprintf (f, "<td>");
+        for (it = list; it != NULL; it = it->next) {
+                shortname = qname_to_shortname ((gchar *)it->data);
+                g_fprintf (f, "%s%s", shortname, (it->next ? ", " : ""));
+                g_free (shortname);
+        }
+        g_fprintf (f, "</td>");
+}
 
 static void
 print_html_header (FILE *f, OntologyDescription *desc)
@@ -60,23 +75,40 @@ print_html_header (FILE *f, OntologyDescription *desc)
         g_fprintf (f,"<body>\n");
         g_fprintf (f,"<div class=\"head\">\n");
         g_fprintf (f," <div class=\"nav\">\n");
-        g_fprintf (f," <a href=\"http://www.maemo.org\"> <img alt=\"MAEMO logo\" ");
-        g_fprintf (f," src=\"../resources/maemo-logo.gif\" /></a>\n");
+
+        /* Three logos at the top. Tracker, maemo, nepomuk */
+        g_fprintf (f, " <a href=\"http://www.tracker-project.org\">");
+        g_fprintf (f, "<img alt=\"Tracker logo\" src=\"../resources/tracker-logo.png\" /></a> \n");
+        g_fprintf (f, " <a href=\"http://www.maemo.org\"> <img alt=\"MAEMO logo\" ");
+        g_fprintf (f, " src=\"../resources/maemo-logo.gif\" /></a>\n");
+        g_fprintf (f, " <a href=\"http://nepomuk.semanticdesktop.org\"> ");
+        g_fprintf (f, "<img alt=\"Nepomuk logo\"  src=\"../resources/nepomuk-logo.png\"/></a>\n");
+
+        g_fprintf (f,"</div>\n");
         g_fprintf (f,"</div>\n");
-// style="border: 0px solid ; width: 180px; height: 88px;"  /> </a> </div>
 
         g_fprintf (f,"<h1>%s</h1>\n", desc->title);
         g_fprintf (f," <dl>\n");
-        g_fprintf (f,"  <dt>Upstream:</dt><dd><a href=\"%s\">Upstream version</a></dd>\n",
-                   (desc->upstream ? desc->upstream : "#"));
+        if (desc->upstream) {
+                g_fprintf (f,"  <dt>Upstream:</dt><dd><a href=\"%s\">Upstream version</a></dd>\n",
+                           desc->upstream);
+        } else {
+                g_fprintf (f,"  <dt>Upstream:</dt><dd>Not available</dd>\n");
+        }
         g_fprintf (f,"  <dt></dt>\n");
         g_fprintf (f,"  <dt></dt>\n");
+        g_fprintf (f, "</dl>\n <dl>\n");
         g_fprintf (f,"  <dt>Authors:</dt>\n");
         g_list_foreach (desc->authors, print_author, f);
+        g_fprintf (f, "</dl>\n <dl>\n");
         g_fprintf (f,"  <dt>Editors:</dt>\n");
         g_list_foreach (desc->editors, print_author, f);
-        g_fprintf (f,"  <dt>Contributors:</dt>\n");
-        g_list_foreach (desc->contributors, print_author, f);
+        if (desc->contributors) {
+                g_fprintf (f, "</dl>\n <dl>\n");
+                g_fprintf (f,"  <dt>Contributors:</dt>\n");
+                g_list_foreach (desc->contributors, print_author, f);
+        }
+        g_fprintf (f, "</dl>\n <dl>\n");
         g_fprintf (f,"  <dt>Changelog:</dt>\n");
         g_fprintf (f,"  <dd><a href=\"%s\">Tracker changes</a>", 
                    (desc->gitlog ? desc->gitlog : "#"));
@@ -159,6 +191,14 @@ print_ontology_class (gpointer key, gpointer value, gpointer user_data)
         g_fprintf (f,"<td>%s</td>\n", (def->description ? def->description : "--"));
         g_fprintf (f,"</tr>\n");
 
+        if (def->instances) {
+                g_fprintf (f,"<tr>");
+                g_fprintf (f,"<td class=\"rowheader\">Predefined instances</td>");
+                print_list (f, def->instances);
+                g_fprintf (f,"</tr>\n");
+        }
+
+
         g_fprintf (f,"</table>\n\n");
 
 }
